/******************************************************************************
;   AIRBOARD Motor Control Board                                              *
;******************************************************************************
;                                                                             *
;    Filename:                                                                *
;    Date: 01/12/04                                                           *
;    File Version: 2.0                                                        *
;                                                                             *
;    Author:  	Crotta Nicola                                                 *
;    			Politecnico di Milano	                              *
;                                                                             * 
;******************************************************************************
;                                                                             *
;    Files required:         P18F452.H                                        *
;                                                                             *
;*****************************************************************************/


#ifndef __DEFINITIONS_H
#define __DEFINITIONS_H

#include <P18F252.H> 				// Define PIC we are using

/*-----------------------------------
----------- DEFINITIONS -------------
-----------------------------------*/

//PIC pins
//#define AI_CURRENT_M0 	PORTAbits.RA0 	//analog in: sense M0 current
//#define AI_CURRENT_M1 	PORTAbits.RA1 	//analog in: sense M1 current
//#define DOUT_LED_1		PORTAbits.RA2	//digital out: status led 1
//#define AI_BATT_SENSE	PORTAbits.RA3	//analog in: sense battery voltage 

//#define DOUT_LED_2		PORTAbits.RA5	//digital out: status led 2


#define DIN_DIR_M0	PORTBbits.RB3	//digital out: direction command M0
#define DIN_DIR_M1	PORTBbits.RB1  	//digital out: direction command M1

#define DOUT_DIR_M0 	PORTBbits.RB2 	//digital out: direction signal M0
#define DOUT_DIR_M1 	PORTBbits.RB0 	//digital out: direction signal M1

/*
#define DOUT_B_MUX_IN	PORTBbits.RB6	//digital out: pin B command multiplexer
#define DOUT_C_MUX_IN	PORTBbits.RB7	//digital out: pin C command multiplexer
*/

#define LED2			PORTBbits.RB5
#define LED1			PORTBbits.RB4

#define DIN_TICKS_M0	PORTAbits.RA4	//digital in: ticks counting encoder M0
#define DIN_TICKS_M1	PORTCbits.RC0	//digital in: ticks counting encoder M1

//#define DOUT_D_MUX_IN	PORTCbits.RC1	//digital out: pin D command multiplexer

#define DOUT_PWM_M0		PORTBbits.RC1  	//digital out: pwm signal M0
#define DOUT_PWM_M1		PORTCbits.RC2  	//digital out: pwm signal M1

#define I2C_SCL			PORTCbits.RC3	//signal I2C: SCL
#define I2C_SDA			PORTCbits.RC4	//signal I2C: SDA

//#define DIN_MUX_OUT		PORTCbits.RC5	//digital in: output multiplexer scan input

/*
#define DOUT_K0			PORTDbits.RD0	//digital out: K0 output
#define DOUT_K1			PORTDbits.RD1	//digital out: K1 output
#define DOUT_K2			PORTDbits.RD2	//digital out: K2 output
#define DOUT_K3			PORTDbits.RD3	//digital out: K3 output
#define DOUT_LED_1		PORTDbits.RD4	//digital out: led
#define DOUT_K9			PORTDbits.RD5	//digital out: K5 output
#define DOUT_K8			PORTDbits.RD6	//digital out: K6 output
#define DOUT_K7			PORTDbits.RD7	//digital out: K7 output
#define DOUT_K6			PORTEbits.RE0	//digital out: K8 output
#define DOUT_K5			PORTEbits.RE1	//digital out: K9 output
#define DOUT_K4			PORTEbits.RE2	//digital out: K10 output
*/
/* Having PWM @ 40KHz, pwm period is 25 us.
We have 40Khz because the max resolution for the PWM that we want is 10 bits.
Now:
	max res = log (Fosc/Fpwm) / log (2); where Fosc is 40MHz = 4*10^7
	10		= log (Fosc/Fpwm) / log (2);
	10 * log (2) = 3;
								Fpwm = Fosc / 10^3 = 4*10^7 / 10^3 = 4*10^4 = 40KHz
Now:
	Tpwm = (PR2 + 1) * 4 * Tosc * TMR2PRESCALER
	and:
		Tpwm = 1/Fpwm
		Tosc = 1/Fosc
		TMR2PRESCALER = 1;
so:
	PR2 = Fosc/(Fpwm*4) - 1
		= 4*10^7/4*10^4 * 1/4 - 1
		= 10^3/4 - 1 
		= 250 - 1 = 249 = 0xF9
*/
#define FOURTY_KHZ  0xF9 //249

#define PWM_ZERO 	0
#define PWM_MAX 	1023
#define PWM_MIN 	0

/* Timer 3: system timer
Fosc/4 = 10MHz = 0.1 us
T3 increments every 0.1*8 us = 0.8 us
1 ms => 1 ms / 0.8 us = 1250 increments for timer 3
2 ms => 1250*2 = ... and so on

Con l'oscilloscopio ho osservato che 1 ms = 1202 incrementi per timer3
2.5 ms = 1202 * 2.5 = 3005 incrementi teorico; sperimentale = 3079

Il valore da assegnare per il restart di timer3 :
65535 - "incrementi":
1 ms 	=> 65535 - 1202 = 64333;
2 ms 	=> 63131
2.5 ms 	=> 65535 - 3005 = 62530;
3 ms 	=> 61929
5 ms 	=> 59525
10 ms	=> 1250*10 => 12500
20 ms   => 1250*20 = 25000
30 ms   => 37500
*/
//Timer 3 1ms value
#define ONE_MS_VALUE 	64333	// calcolato sperimentalmente con l'oscilloscopio
#define TIME_VALUE		65535-12500

//engines definitions
#define ENGINE_0	0				//define engine0 value
#define ENGINE_1	1  				//define engine1 value
#define BOTH_ENGINES 2 				//define both engines: used when we want to set or use both engines

#define TEST_COPPIA 3				//define test per la coppia con pesi

//engines status
#define ENGINE_ON	!ENGINE_OFF		//define engine status on
#define ENGINE_OFF 0				//define engine status off

//engines brake
#define ENGINE_GO	!ENGINE_BRAKE 	//define engine movement
#define ENGINE_BRAKE 0				//define engine brake

//engines direction
#define ENGINE_FORWARD 1				//define engine forward direction
#define ENGINE_BACKWARD 0 				//define engine backward direction

//kickers definitions
#define KICKER_0 0
#define KICKER_1 1
#define KICKER_2 2
#define KICKER_3 3

//ADC definitions
#define M0_CURRENT 	0
#define M1_CURRENT 	1
#define BATTERY		2

//command mode definition
#define SPEED 		0
#define CURRENT	 	1

#define FALSE 0
#define TRUE !FALSE

#define OFF 0
#define ON !OFF

#define OK 1
#define ERROR !OK

#define DEBUG_DEFAULT 0
#define DEBUG_SMALL 1
#define DEBUG_MEDIUM 2
#define DEBUG_FULL 3

#define DEBUG_COUNTER_DELAY 500				//ritardo messaggio debug (* 1ms)
#define INPUT_COUNTER_DELAY 100				//ritardo lettura ingressi digitali (* 1ms)

#define MAX_TEST 10

/*-----------------------------------
--------ERRORS CODE -----------------
-----------------------------------*/
#define UKNOWN_CMD 	99	
#define BAD_DEVC 	2
#define BAD_DATA 	3
#define BUSY 		4	
#define M0_DISABLE	5
#define M1_DISABLE	6
#define BAD_CMD 	7

/*-----------------------------------
--------EEPROM ADDRESS ----------
-----------------------------------*/

#define SIGN_EXP 1 		//number of bytes reserved for the sign of the exponent
#define EXP 3 			//number of bytes reserved for the exponent value. MAX exponent is 2^EXP
#define VALUE 12 		//number of bytes reserved for the value. MAX number is 2^VALUE

#define POSITIVE_EXP 0 	//exponent is positive
#define NEGATIVE_EXP 1 	//exponent is negative

/*Here you can define pic internal eprom addresses for important variables storing.
The space needed for each variable is two eeprom memory cells.
The variable is stored in the follow way:
SIGN_EXP bits for sign of the exponent
EXP bits for the exponent
VALUE bits for the value
*/

/*PID proportional constant for engine 0.*/
#define KP0_ADDR 10

/*PID integral constant for engine 0.*/
#define KI0_ADDR 12

/*PID derivative constant for engine 0.*/
#define KD0_ADDR 14

/*PID proportional constant for engine 1.*/
#define KP1_ADDR 16

/*PID integral constant for engine 1.*/
#define KI1_ADDR 18

/*PID derivative constant for engine 1.*/
#define KD1_ADDR 20

/*Sampling period. Sampling period is in milliseconds (ms)*/
#define CT_ADDR 22

/*Kick delay.(ms)*/
#define CK_ADDR 24

/*Max tick for engine 0.*/
#define MAXTC0_ADDR 26

/*Max tick for engine 1.*/
#define MAXTC1_ADDR 28

/*Tick to space for engine 0.(cm/impulsi)*/
#define TICSP0_ADDR 30

/*Tick to space for engine 1.*/
#define TICSP1_ADDR 32

/*Minimun battery voltage (unit 0,00 V) */
#define BATT_VOLT_LIMIT_ADDR 34  

/*ADC read delay (unit ms)*/
#define ADC_DELAY_ADDR 36

/*Command mode for motors */
#define COMMAND_MODE_ADDR 38

/*Max current for engine 0*/
#define MAX_CURRENT_M0_ADDR 40

/*Max current for engine 1*/
#define MAX_CURRENT_M1_ADDR 42

/*Max current for engine 1*/
#define SETPOINT_ADDR 42

/*Boot mode */
#define BOOT_MODE_ADDR 255

/*-----------------------------------
--------ENCODERS VARIABLES ----------
-----------------------------------*/
/*
extern long total_tick_0; 			//engine 0 total tick
extern long total_tick_1; 			//engine 1 total tick
extern int tick_0;					//engine 0 tick
extern int tick_1;					//engine 1 tick
extern unsigned int encoder_read_0, encoder_read_1;
extern int setpoint_0, setpoint_1;
extern long already_read_ticks_0, already_read_ticks_1;
extern unsigned int max_tick_0, max_tick_1;
extern unsigned float ticks_to_speed_0, ticks_to_speed_1;
extern unsigned float ticks_to_space_0, ticks_to_space_1;
*/
/*-----------------------------------
------------ CONTROL ----------------
-----------------------------------*/
/*
extern unsigned char command_mode;
extern int control_counter;
extern unsigned int battery_counter;
extern int current_speed_0, current_speed_1;
extern unsigned float kp_0, kp_1, kd_0, kd_1, ki_0, ki_1;
extern int	y_0, y_1; 
extern int u_0, u_1; 
extern unsigned int debug_counter, input_counter;
extern unsigned long debug_sample;
extern unsigned float ticks_to_pwm_0, ticks_to_pwm_1;
extern unsigned float current_to_pwm_0, current_to_pwm_1;
extern unsigned char debug;
extern unsigned char tuning;
extern int m0_current, m1_current;
extern unsigned int current_limit;
extern unsigned int m0_max_current, m1_max_current;
*/

/*-----------------------------------
------- CUMMUNICATION BUFFER --------
-----------------------------------*/
#define DIM_INPUT_BUFFER 4 		//input buffer size
#define DIM_SERIAL_WRITE 2 		//buffer used when writing to serial

#define VARIABLE_SIZE 16
#define VALUE_SIZE 16


extern char parsingInputBuffer[]; 	// Input buffer
//extern char inputBuffer[];
extern unsigned char dataReady; 	//TRUE if command is ready to be execute
extern unsigned char byteReceived; 	//number of byte received
//extern unsigned char Command; 		//command to execute
//extern unsigned char Device; 		//device to control
//extern unsigned char sign; 			//sign to control
//extern int InputData0; 				//input data 0
//extern int InputData1; 				//input data 1


/*-----------------------------------
--------- OTHERS VARIABLES ----------
-----------------------------------*/
//extern char string_temp[40];
extern unsigned char ADC_select;


/* V A R I A B I L I   E   C O S T A N T I   A G G I U N T E   D A   M E   (P U N T O   1)*/

#define LOR_ZERO			650.0

#define LV_ZERO		7.8	
#define OMEGA_ZERO		510
#define LV_M			0.0029
#define LOR_M			0.0075

/*****
Sensibilita' sensore: 40 mV/s
Escursione segnale: 4.8 V (0->4.8V)

Costante di proporzionalit omega:
(4.8/0.04)/1023*pi/180 = -0.00204730704046256
(4/0.04)/1023*pi/180 = -0.00170608920038546
(5/0.04)/1023*pi/180 = -0.00213052887206339

-01		5/1024*40*10^ -3   0.0001953125
-02		5/1024*10*10^ -3   0.000048828125
-75		5/1024*27*10^ -3   0.0001318359375
*****/

#define OMEGA_M			-0.001

#define TICK_PER_TURN	16896

// 2*pi*0.075/16896
#define TICK_TO_METER	0.0000278905597797389*100.0

#define G				9.80665

//#define BARICENTRO		0.1896   //INUTILE
#define POSIZIONE_ADXL	0.075		//INUTILE

//1023.0/Cmax 	//INUTILE
//1023.0/7.5 = 136.4	//INUTILE
//1023.0/7.0 = 146.14	//INUTILE
//1023.0/4.5 = 227.33	//INUTILE
#define K_SCALE 		3000 //90

#define BUFFER_SIZE		8
#define BUFFER_SHIFT    3
#define W_SIZE 33
#define W_SHIFT 5

#define W_SIZE2 8
#define W_SHIFT2 3	
// 1/20ms = 1000/20 = 50	//INUTILE
// 1/10ms = 100				//INUTILE
#define INT_DER			100

// costanti per calcolo osservatore ridotto   //INUTILE
#define A				0.92311
#define Bcoppia         -0.077748
#define Btheta			-0.012500

extern float theta;				// valore dell'angolo di inclinazione  //INUTILE
extern int vError;				// errore di velocit (differenza tra velocit dei motori e profilo)

#define SLAVE_ADD 0b11100110

// *****************************

// COSTANTI CHE INFLUENZANO IL COMPORTAMENTO

//#define STILL // non muove i motori
#define SERIAL_DEBUG //permette acquisizione dei valori tramite seriale (4.2 ms con invio a 76800, 1.1 ms senza invio di dati)
//#define PROFILE // esegue profilo di velocit
//#define I2C_SEND
//#define TEST1 //test a ruote bloccate, per la stima dell'angolo
#define TEST2 //test dinamico con piu' filtri per l'angolo
// *****************************

/* E N D */
#endif
