/***********************
*******************************************************
;   AIRBOARD Motor Control Board                                              *
;******************************************************************************
;                                                                             *
;    Filename: ROBOT_MAIN.C                                                   *
;    Date: 01/12/04                                                           *
;    File Version: 2.1                                                        *
;                                                                             *
;    Author:  2004:      Doni Stefano, Ferroni Alessandro, Crotta Nicola      *
;			  2006/2007: Antolini Michele, Bolognino Marco                    *
;                                                                             * 
;******************************************************************************
;                                                                             *
;    Files required: DEFINITIONS.H, 								          *                                                                 
;*****************************************************************************/

//Bit di configurazione microcontrollore

#pragma config OSC = HSPLL, OSCS = OFF		//Quarzo funzionamento HS-PLL -> 40 MHz. Nessun cambio oscillazione
#pragma config PWRT = OFF					//Power-up timer non utilizzato
#pragma config BOR = OFF					//Brown-out reset non utilizzato
#pragma config WDT = OFF,  WDTPS = 1	//EDITAREEE	//Watch-dog timer attivo
#pragma config CCP2MUX = ON				//Nessun multiplexer periferica capture-compare
#pragma config LVP = OFF					//Low-Voltage Programming non abilitato
#pragma config DEBUG = ON

#include <adc.h>
#include <definitions.h>
#include <delays.h>
#include <float.h>
#include <i2c.h>
#include <math.h>
#include <p18f252.h>
#include <pwm.h>
#include <rs232.c>
#include <stdlib.h>
#include <timers.h>
#include <usart.h>

//Versione firmware
#define VERSION 3

// finestra per media mobile
float f1W[W_SIZE];
// cursore
unsigned char f1Wc;
// accumulatore 
float f1Wa;

char parsingInputBuffer[DIM_INPUT_BUFFER]; 		//input buffer for string parsing
char inputBuffer[DIM_INPUT_BUFFER];				//input buffer string receiving		
unsigned char dataReady=0,inputBufferCounter=0, Command=0, Device=0, byteReceived = 0;
int InputData0=0, InputData1=0; 				//rs232 input data
long total_tick_0=0, total_tick_1=0; 			//engine tick counters
unsigned char command_mode=0;					//command mode for motors (=0 speed, =1 current)
int control_counter=0; 							//control delay counter 
unsigned int ADC_counter=0;						//ADC delay counter
unsigned int control_kick=0; 					//control kick conter
int setpoint_0=0, setpoint_1=0; 				//engine 0 and engine 1 setpoints
int current_speed_0=0, current_speed_1=0; 		//engine 0 and engine 1 current speed
int m0_current=0, m1_current=0;  //m0 and m1 current
unsigned int m0_max_current=0, m1_max_current=0;//max current for each motor
unsigned char sign=0;							//define sign for setpoint value
float theta = 0;		// valore dell'angolo di inclinazione (stimato da ADXL202)
float coppia;				// valore di indice dell'array di coppiapwm
float omega;
int pwm;				// pwm da assegnare ai motori
short pwmsign;			// segno del pwm in uscita
int adxlCounter = 0;
//utili per controllare i valori letti in fase di debug
int adxlBuffer[BUFFER_SIZE];
int tdIndex;
int adxlIndex;
int adxlSum;
int wdtCount;
int wdtDelay;
int tick;
float vel, velbis;
int tempo;
float lor, lv;
// valori acquisiti senza normalizzazione
int lor_raw, lv_raw, omega_raw;
float controllo;
unsigned int controllo_i;
int blsIndex;
float integr;
//Variabili locali 
unsigned char m0_enable=FALSE, m1_enable=FALSE;	
unsigned int ADC_counter_delay=0;

#ifdef TEST2
float thetaNoFilt = 0;
float thetaFilt = 0;
float thetaSin = 0;
float root;
int acc;

rom float bls[W_SIZE] = {  
   0.014307198868959,
   0.017364998689464,
   0.020400234905321,
   0.023383581871452,
   0.026285918314030,
   0.029078659261423,
   0.031734083584860,
   0.034225653095555,
   0.036528319237561,
   0.038618813559597,
   0.040475918342816,
   0.042080714002401,
   0.043416800166037,
   0.044470487657892,
   0.045230958978678,
   0.045690395265806,
   0.045844068137524,
   0.045690395265806,
   0.045230958978678,
   0.044470487657892,
   0.043416800166037,
   0.042080714002401,
   0.040475918342816,
   0.038618813559597,
   0.036528319237561,
   0.034225653095555,
   0.031734083584860,
   0.029078659261423,
   0.026285918314030,
   0.023383581871452,
   0.020400234905321,
   0.017364998689464,
   0.014307198868959};
#endif




//Prototipi di funzione
void setup(void);
void control_isr(void);


/*-------------------------------------------------------------------------------
----------- INTERRUPT A PRIORITA' BASSA- CONTROLLO ------------------------------
-------------------------------------------------------------------------------*/

#pragma interruptlow control_isr
#pragma code IT_vector_low=0x18
void Interrupt_low_vec(void)
{
   _asm goto control_isr  _endasm
}
#pragma code
void control_isr() {

	if (PIR2bits.TMR3IF) {					//interrupt 20ms

		LED2 = 1;
		WriteTimer3(TIME_VALUE);			//Restart Timer3
		// selezione del canale 02 dell'ADC per la lettura del sensore ADXL202
		SetChanADC(ADC_CH0);
		Delay10TCYx(6);						// Ritardo predisposizione lettura mxu ingresso analogico
		adxlSum = 0;
        adxlCounter = 0;
		for (adxlCounter = 0; adxlCounter < BUFFER_SIZE; adxlCounter++) {
			ConvertADC();						// inizia la conversione
			while(BusyADC());					// attendi la fine della conversione
			;
			adxlSum += ReadADC(); 				// somma dei valori
		}
		// calcolo del valore dell'ADXL202 = adxlSum / BUFFER_SIZE; non tolgo OFFSET
		lor_raw = adxlSum>>BUFFER_SHIFT;
		lor = ((float)adxlSum/(float)BUFFER_SIZE) - LOR_ZERO;
		// calcolo il lor reale m/(s^2)
		lor *= LOR_M;	
		// selezione del canale dell'ADC
		SetChanADC(ADC_CH1);
		Delay10TCYx(6);						// Ritardo predisposizione lettura mxu ingresso analogico
        adxlSum = 0;
        adxlCounter = 0;
		for (adxlCounter = 0; adxlCounter < BUFFER_SIZE; adxlCounter++) {
			ConvertADC();						// inizia la conversione
			while(BusyADC());					// attendi la fine della conversione
			adxlSum += ReadADC(); 				// somma dei valori
		}
		lv_raw = adxlSum>>BUFFER_SHIFT;
		lv = ((float)adxlSum/(float)BUFFER_SIZE);
		// calcolo lv reale m/(s^2)
		lv *= LV_M;
		lv = lv + LV_ZERO;
		// selezione del canale dell'ADC
		SetChanADC(ADC_CH2);
		Delay10TCYx(6);						// Ritardo predisposizione lettura mxu ingresso analogico
        adxlSum = 0;
        adxlCounter = 0;
		for (adxlCounter = 0; adxlCounter < BUFFER_SIZE; adxlCounter++) {
			ConvertADC();						// inizia la conversione
			while(BusyADC());					// attendi la fine della conversione
			adxlSum += ReadADC(); 				// somma dei valori
		}
		omega_raw = adxlSum>>BUFFER_SHIFT;
		omega = ((float)adxlSum/(float)BUFFER_SIZE) - OMEGA_ZERO;
		omega *= OMEGA_M;
		LED2 = 0;
/*************************** LETTURA VELOCITA' *****************************/
#ifndef TEST1
#ifndef TEST2
		// leggo i tick percorsi fino ad ora dal reset precedente
		tick = ReadTimer1()*((DIN_DIR_M1==0) ? 1 : -1);
		// reset timer3
		WriteTimer1(0);
		vel=tick;
#endif
#endif

#ifdef TEST1
		tick = ReadTimer1()*((DIN_DIR_M1==0) ? 1 : -1);
		WriteTimer1(0);
		vel = vel + tick;		
#endif

#ifdef TEST2
		acc = tick;
		tick = ReadTimer1()*((DIN_DIR_M0==0) ? 1 : -1);
		WriteTimer1(0);
		vel=tick;
		acc = tick - acc;
#endif
/**************************** STIMA DI THETA *******************************/
#ifdef TEST1		
		if (( fabs(theta) < 0.0017) && LED1 == 0 ) //se l'angolo  < di 0.0017 si accende il led 1 e resetta vel
		{
			vel=0;
			LED1=1;
		}
#endif

#ifdef TEST2
		root = lor*lor + lv*lv - 96.1704;
		if (root < 0.0) root = 0.0;
		// scegliamo la soluzione in base al verso dell'accelerazione
		if (acc >= 0) {
			thetaNoFilt = 2.0*atan((lor-sqrt(root))/(lv+9.8066));
		} else {
			thetaNoFilt = 2.0*atan((lor+sqrt(root))/(lv+9.8066));
		}
		f1W[f1Wc] = thetaNoFilt;
		f1Wa = 0.0;
		for(blsIndex = 0; blsIndex < W_SIZE; blsIndex++)
		{
			f1Wa += bls[blsIndex] * f1W[(f1Wc+blsIndex)%W_SIZE];
		}
		f1Wc = ++f1Wc%W_SIZE;
		thetaFilt = f1Wa;
		
		thetaSin= asin ( lor / G);
#endif	
		integr = omega * 0.02 ;
		theta = 0.98 * ( theta + integr ) + 0.02 * ( lor ) ;
		pwm = (int)(theta*K_SCALE);
		if ( pwm > 1023 ) pwm = 1023;		
		if ( pwm < -1023 ) pwm = -1023;
#ifndef PROFILE
		// assegnamento del valore di pwm ai motori
		if (pwm < 0.0)     	//	if (coppia < 0.0)
		{
 	#ifndef STILL
			DOUT_DIR_M0 = ON;
			DOUT_DIR_M1 = ON;

			SetDCPWM1(-pwm);		
			SetDCPWM2(-pwm); 
  	#endif
		}
		else
		{
	 #ifndef STILL
			DOUT_DIR_M1 = OFF;
			DOUT_DIR_M0 = OFF;

			SetDCPWM1(pwm);
			SetDCPWM2(pwm); 
  	#endif
  		}
#endif    

#ifdef TEST2
//print dei valori via seriale
		LED1 = 1;
		putiUSART(0xFFFF);
		putfUSART(lor);
		putfUSART(lv);
		putfUSART(omega); 
		putfUSART(thetaNoFilt);  
		putfUSART(thetaFilt);    
		putfUSART(thetaSin);   
		putfUSART(theta);
		putiUSART(pwm);
		LED1 = 0;	 

#endif

#ifdef SERIAL_DEBUG
#ifndef TEST2
//print dei valori via seriale
		LED1 = 1;
		putiUSART(0xFFFF);
		putfUSART(lor);
		putfUSART(lv);
		putfUSART(vel); //*TICK_TO_METER);    //velocit [m/s]
		putfUSART(omega);  //velocit angolare
		putfUSART(theta);    //theta tiltrato
		putfUSART(coppia);   //   VA TOLTOOOOOOO!!
		putfUSART(thetaNoFilt); // theta non filtrato  VA TOLTOOOOOOO!!
		putiUSART(pwm);
		LED1 = 0;	 

#endif
#endif

//CONTROLLARE COSA DIAVOLO SIA
#ifdef I2C_SEND	
				
		IdleI2C();
		StartI2C();
		IdleI2C();
		WriteI2C(SLAVE_ADD);
		IdleI2C();
		IdleI2C();
		StopI2C();	
	
#endif
		//azzera flag overflow Timer3
		PIR2bits.TMR3IF = 0;
	}

}
#pragma code

/*-------------------------------------------------------------------------------
------------------------- COMMAND EXECUTION -------------------------------------
-------------------------------------------------------------------------------*/

//retrieveInputData preleva i dati dalla stringa costruita via seriale
//len  la lunghezza della stringa e offset  l'indice della stringa
unsigned char retrieveInputData(unsigned char input, unsigned char len, unsigned char offset) {
	unsigned char signOffset=1;
	char tempBuffer[9]; //temporary buffer for character storage
	int inputData;
	unsigned char i;
	
	if (len>9)
		return ERROR;
	
	sign = parsingInputBuffer[offset];
	if (sign != '+' && sign != '-') {
		if (sign<'0' || sign>'9')
			return ERROR;
		else
			signOffset=0;
	}
   
   	//retrieve inputdata
   	for (i=0; i<len; i++) {
   	tempBuffer[i] = parsingInputBuffer[offset+i+signOffset];
   	if (tempBuffer[i]<'0' || tempBuffer[i]>'9')
   		return ERROR;
   	}
   	tempBuffer[len] = '\0';
	
 	//convert temp buffer to number
   	inputData = atoi(tempBuffer);
   
   	//copy value to right input data 
   	if (input == 0) 
   		InputData0 = inputData;
   	else if (input == 1) 
   		InputData1 = inputData;
   	else 
   		return ERROR;
   	//operation ok
   	return OK;
}


/*-------------------------------------------------------------------------------
----------- PIC SETUP -----------------------------------------------------------
-------------------------------------------------------------------------------*/

//PIC setup
void Setup(void) {
	unsigned int i; //local counter variable

	// Reset interrupt
	INTCON = INTCON2 = INTCON3 = 0;
	PIR1 = 0;
	PIR2 = 0;
	PIE1 = 0;
	PIE2 = 0;
	IPR1 = 0;
	IPR2 = 0;

	//Configure ADC
	/*
		ADC clock = FOSC/64
		ADC right justification
		ADC input and voltage ref: 5 analog inputs and 0 ref,
		ADC channel CH0 select first
		ADC interrupt off
	*/
	OpenADC(ADC_FOSC_64 & ADC_RIGHT_JUST & ADC_5ANA_0REF, ADC_CH0 & ADC_INT_OFF);

	//enable PORTA
	/*
	RA0: IN ADC accellerometro microchip
	RA1: IN ADC accellerometro microchip
	RA2: IN ADC accellerometro angolare
	RA3: IN ADC accellerometro ST
	RA4: IN Encoder tiks M0 al momento
	RA5: IN ADC accellerometro ST
	*/

	PORTA = 0x00;		 //Clear PORTA
	TRISA = 0b111111; 	 

	//enable PORTB
	/*
	RB0: OUT direction mot1
	RB1: IN direction mot1 
	RB2: OUT direction mot2
	RB3: IN direction mot2
	RB4: OUT Led 1
	RB5: OUT Led 2
	BR6: Per programmazione PGC
	RB7: per programmazione PGD
	*/

	PORTB = 0x00;		//Clear PORTB
	TRISB = 0b11001010;	
	
	//enable PORTC
	/*
	C0: IN Encoder tick M1 al momento
	C1: OUT PWM1
	C2: OUT PWM2
	C3
	C4
	C5:
	C6: OUT USART
	C7: IN USART
	*/

	PORTC = 0x00; 		//Clear PORTC
	TRISC = 0b11000001; 
	
	//configure Timer0: engine 0 tick counter
	/*
		Timer0 interrupt off
		Timer0 at 16 bit
		Timer0 external source
		Timer0 ad edge fall
		Timer0 prescaler 1:1	
	*/
	OpenTimer0(TIMER_INT_OFF & T0_16BIT & T0_SOURCE_EXT & T0_EDGE_FALL & T0_PS_1_1);
	WriteTimer0(0);	

	//configure Timer1: engine 1 tick counter
	/*
		Timer1 interrupt off
		Timer1 at 16 bit with less significative bit right
		Timer1 external source
		Timer1 prescaler 1:1
		Timer1 oscillator off
		Timer1 external synch off
	*/
	OpenTimer1(TIMER_INT_OFF & T1_16BIT_RW & T1_SOURCE_EXT & T1_PS_1_1 & T1_OSC1EN_OFF & T1_SYNC_EXT_OFF);
	WriteTimer1(0);
	
	//configure Timer2: PWM generator
	/*
		Timer2 interrupt off
		Timer2 prescaler 1:1
		Timer2 postscaler 1:1
	*/
	OpenTimer2(TIMER_INT_OFF & T2_PS_1_1 & T2_POST_1_1);
			
	//configure Timer3: 1ms interrupt on
	/*
		Timer3 interrupt on
		Timer3 at 16 bit with less significative bit right
		Timer3 internal source
		Timer3 prescaler 1:8
		Timer3 external synch off.
	*/
	OpenTimer3(TIMER_INT_ON & T3_16BIT_RW & T3_SOURCE_INT & T3_PS_1_8 & T3_SYNC_EXT_OFF );

	//configure PWM1 and PWM2 frequency at 40KHz
	OpenPWM1(FOURTY_KHZ);
	OpenPWM2(FOURTY_KHZ);

	//configure USART 
	/*
		Value to use for SPBRG
		129 for baud rate = 19200 bit/s
         42 for baud rate = 57600 bit/s
		 32 for baud rate = 76800 bit/s
		 20 for baud rate = 115200 bit/s
	*/
	
	SPBRG = 20; //baud rate
	
	TXSTA = 0x24; //transmit register
	RCSTA = 0x90; //receive register
	PIE1bits.RCIE = TRUE; //enable receive interrupt
	PIR1bits.TXIF = 0;
	
	
	OpenUSART(
			USART_TX_INT_OFF &
			USART_RX_INT_OFF &
			USART_ASYNCH_MODE & 
			USART_EIGHT_BIT & 
			USART_SINGLE_RX &
			USART_BRGH_HIGH,
			20
	);
 //USART DA RIABILITARE
//	debug = 0;
	setpoint_0 = 0;
	setpoint_1 = 0;

/*
	//reset input buffer
	for (i=0; i<DIM_INPUT_BUFFER; i++){ 
		parsingInputBuffer[i]=0;
		inputBuffer[i]=0;		
	}
*/
	//reset serial bus variables 
	dataReady = 0;
	Command = 0;
	Device = 0;
	byteReceived = 0;
	
	m0_enable=FALSE;
	m1_enable=FALSE;

	current_speed_0 = 0;
	current_speed_1 = 0;

	control_counter = 0;	
	ADC_counter_delay  = 3; // CAMPIONAMENTO OGNI 5ms
	
	adxlCounter = 0;
	
// utili per controllare i valori letti in fase di debug
//	for (tdIndex = 0; tdIndex < BUFFER_SIZE; tdIndex++)
//		tdBuffer[tdIndex] = 0;


	for (f1Wc = 0; f1Wc < W_SIZE; f1Wc++)
		f1W[f1Wc] = 0;

	f1Wc = 0;
	f1Wa = 0;
	adxlIndex = 0;
	tdIndex = 0;
	adxlSum = 0;
	wdtCount = 0;
	tick = 0;
	tempo = 0;
	pwmsign = 0;
	pwm = 0;
	omega = 0.0;
	theta = 0.0;
	DOUT_DIR_M0 = OFF;
	DOUT_DIR_M1 = OFF;
	vel=0;
	LED1=0;

	//configure interrupt priorities
	IPR1bits.RCIP = 1; 				//USART receive interrupt priority high
	IPR2bits.TMR3IP = 0; 			//TMR2 interrupt priority low
	IPR1bits.ADIP = 0;				//ADC interrupt priority low
	RCONbits.IPEN = 1; 				//enable interrupt priorities	
	
	//enable interrupts
	INTCONbits.PEIE = 1;			// Enable peripheral interrupts
	INTCONbits.GIEH = 1; 			// Enable all interrupts

	controllo = -5.0;
	controllo_i = 0;
	/* Inizializzazione I2C master */

#ifdef I2C_SEND

	I2C_SCL=1;			// Impostazione linee SCL/SDA come input
	I2C_SDA=1;
	SSPADD=255;			// Clock a  1 Mhz con Fosc a 20 Mhz
	SSPCON1=0b00111000;	// Modalita' Master, Clock=Fosc/(4*(SSPADD+1))
	SSPCON2=0b11100000;
	SSPSTATbits.CKE=0;  		// Sample mode conforme a standard I2C
	OpenI2C( MASTER, SLEW_OFF );
#endif	
}

/*-------------------------------------------------------------------------------
----------- MAIN PROGRAM -----------------------------------------------------------
-------------------------------------------------------------------------------*/

//main 
void main(void) {

	ClrWdt();
	//Ritardo stabilizzazione tensione e oscillazione
	//	Delay10TCYx(25);
	//Inizializza periferiche micro
	Setup();
	
	//Restore default data from eeprom
	//	restoreData(); 

	//restart watchdog
	ClrWdt();
	
	//Write Timer 3 to obtain 1ms
	WriteTimer3(TIME_VALUE);
	DOUT_DIR_M1 = ON;
	DOUT_DIR_M0 = ON;
	SetDCPWM1(0);
	SetDCPWM2(0);

	pwmsign = 0;
			
	while(1);
}








/*
//	Prova di accellerazione lineare dei motori

	adc_or=-523;
	while(adc_or < 1023){

		adc_or = adc_or + 5,
		Delay10KTCYx(100);
		if ( pwm > 1023 ) pwm = 1023;			
		if (adc_or > 0.0) {
			DOUT_DIR_M0 = ON;
			DOUT_DIR_M1 = ON;
			SetDCPWM1(adc_or);
			SetDCPWM2(adc_or);
			}
		else
			{
		if ( pwm < -1023 ) pwm = -1023;
			DOUT_DIR_M1 = OFF;
			DOUT_DIR_M0 = OFF;
			SetDCPWM1(-adc_or);
			SetDCPWM2(-adc_or);
		}
	}
	SetDCPWM1(0);
	SetDCPWM2(0);
	while(1);
} 
*/