/******************************************************************************
;   AIRBOARD Motor Control Board                                              *
;******************************************************************************
;                                                                             *
;    Filename: RS-232.C		                                                  *
;    Date: 01/12/04                                                           *
;    File Version: 2.0                                                        *
;                                                                             *
;    Author:  2004: Doni Stefano, Ferroni Alessandro, Crotta Nicola                 *
;             2006/2007: Antolini Michele, Bolognino Marco                                                            * 
;******************************************************************************
;                                                                             *
;    Files required: STDLIB.H, PORTB.H, TIMERS.H, DELAY.H, PWM.H, ADC.H       *
;					 DEFINITIONS.H, CONTROLLER.H, RS232.H, EEPROM.H           *                                                                 
;******************************************************************************
;																			  *
;RS232.C																	  *
;RS232 module.																  *
;																			  *
;*****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <usart.h>
//#include "rs232.h"
#include "definitions.h"

// write on rs232 an int
void putiUSART(int i) {

	union { char c[2]; int i; } sendint;
	sendint.i = i;
	while(!(TXSTA & 0x02));
	TXREG = sendint.c[0];
	while(!(TXSTA & 0x02));
	TXREG = sendint.c[1];
}

// write on rs232 a float
void putfUSARTraw(float f) {

	union { char c[4]; float f; } sendfloat;
	sendfloat.f = f;

	while(!(TXSTA & 0x02));
	TXREG = (sendfloat.c[1]&0x80) | (sendfloat.c[0] >> 1);
	while(!(TXSTA & 0x02));
	TXREG = (sendfloat.c[1]&0x7F) | (sendfloat.c[0] << 7);
	while(!(TXSTA & 0x02));
	TXREG = sendfloat.c[2];
	while(!(TXSTA & 0x02));
	TXREG = sendfloat.c[3];
}

void putfUSART(float f) {
	union { char c[4]; float f; } sendfloat;
	sendfloat.f = f;

 	while( BusyUSART() );
	WriteUSART( sendfloat.c[0] );
	while( BusyUSART() );
	WriteUSART( sendfloat.c[1] );
	while( BusyUSART() );
	WriteUSART( sendfloat.c[2] );
	while( BusyUSART() );
	WriteUSART( sendfloat.c[3] );
}	

//write on rs232 a string
void putsUSART(char *data) {
	do
	{
		while(!(TXSTA & 0x02)); //wait for busy rs232
		TXREG = *data; //copy the data into transmission register
	} while( *(++data) );
}
//read a single byte (character) from rs232
char ReadUSART(void) {
	return RCREG; //return reception register
}

//write a single byte (character) on rs232
void WriteUSART(char c) {
	TXREG = c; //put character into transmission register
	//DOUT_LED_1 = DOUT_LED_1^1;
}

/*subroutines called by rs232 interrupt service routine.
It is called when a byte arrives on the serial bus.*/
void read_rs232() {
	static char inputBuffer[DIM_INPUT_BUFFER];
	static unsigned int inputBufferCounter = 0;
	char i;	

	//read a single character from serial bus
	inputBuffer[inputBufferCounter] = ReadUSART();

	//increase bytes number
	inputBufferCounter++;

	if (inputBufferCounter >= DIM_INPUT_BUFFER) {
		//buffer has reached its maximum capacity: counter and command buffer data must be reset
		inputBufferCounter = 0;
	}
  	if (inputBuffer[inputBufferCounter-1] == '\r') {
	  	//insert string terminator
	  	inputBuffer[inputBufferCounter-1]='\0';

  		// copy inputBuffer into parsingInputBuffer
  		/*When we have a CR from serial, we copy the received buffer (inputBuffer)
		to this buffer (parsingInputBuffer) and we start to parse this one*/
  		memcpy((char *)parsingInputBuffer, (const char *)inputBuffer, inputBufferCounter-1);

  		parsingInputBuffer[inputBufferCounter-1]='\0';

   		//decrease bytes received because \r is not a valid byte for the command
   		byteReceived = inputBufferCounter-1;

    	//reset counter
    	inputBufferCounter = 0;
		for(i=0; i < DIM_INPUT_BUFFER; i++)
			inputBuffer[i]=0;

    	//set dataReady to true
 		dataReady = 1;
	}
}




